<?php
/*
Plugin Name: Falling Emojis
Description: Add a fun falling emojis effect across your website screen.
Version: 1.1.1
Author: Tyson Brooks
Author URI: https://frostlineworks.com
Tested up to: 6.3
*/

// Prevent direct access to this file
if (!defined('ABSPATH')) {
    exit;
}

// Add a "Settings" link to the plugin actions in the plugins list
add_filter('plugin_action_links_' . plugin_basename(__FILE__), function ($links) {
    $pluginSlug = basename(dirname(__FILE__)); // Dynamically get the plugin slug
    $settings_link = '<a href="admin.php?page=' . $pluginSlug . '">Settings</a>';
    array_unshift($links, $settings_link); // Add the settings link at the beginning
    return $links;
});

// Ensure the FLW Plugin Library is loaded before running the plugin
add_action('plugins_loaded', function () {

    // Check if the FLW Plugin Library is active
    if (class_exists('FLW_Plugin_Update_Checker')) {
        $pluginSlug = basename(dirname(__FILE__)); // Dynamically get the plugin slug

        // Initialize the update checker
        FLW_Plugin_Update_Checker::initialize(__FILE__, $pluginSlug);

        // Replace the update icon
        add_filter('site_transient_update_plugins', function ($transient) {
            if (isset($transient->response)) {
                foreach ($transient->response as $plugin_slug => $plugin_data) {
                    if ($plugin_slug === plugin_basename(__FILE__)) {
                        $icon_url = plugins_url('assets/logo-128x128.png', __FILE__);
                        $transient->response[$plugin_slug]->icons = [
                            'default' => $icon_url,
                            '1x' => $icon_url,
                            '2x' => plugins_url('assets/logo-256x256.png', __FILE__),
                        ];
                    }
                }
            }
            return $transient;
        });
    } else {
        add_action('admin_notices', function () {
            $pluginSlug = 'flwpluginlibrary/flwpluginlibrary.php';
            $plugins = get_plugins();

            if (!isset($plugins[$pluginSlug])) {
                echo '<div class="notice notice-error"><p>The FLW Plugin Library is not installed. Please install and activate it to enable update functionality.</p></div>';
            } elseif (!is_plugin_active($pluginSlug)) {
                $activateUrl = wp_nonce_url(
                    admin_url('plugins.php?action=activate&plugin=' . $pluginSlug),
                    'activate-plugin_' . $pluginSlug
                );
                echo '<div class="notice notice-error"><p>The FLW Plugin Library is installed but not active. Please <a href="' . esc_url($activateUrl) . '">activate</a> it to enable update functionality.</p></div>';
            }
        });
    }

    // Register the plugin functionality
    if (class_exists('FLW_Plugin_Library')) {
        class FallingEmojis_Plugin {

            /**
             * Constructor to initialize the plugin.
             */
            public function __construct() {
                add_action('wp_enqueue_scripts', [$this, 'enqueue_scripts']);
                add_action('admin_menu', [$this, 'register_submenu']);
                add_action('admin_init', [$this, 'register_settings']);
                add_action('wp_ajax_falling_emojis_settings', [$this, 'handle_ajax_settings']);
                add_action('wp_ajax_nopriv_falling_emojis_settings', [$this, 'handle_ajax_settings']);
            }

            /**
             * Enqueue necessary scripts and styles for falling emojis effect.
             */
            public function enqueue_scripts() {
                // Get the plugin version dynamically from the plugin header
                $plugin_data = get_file_data(__FILE__, ['Version' => 'Version']);
                $plugin_version = $plugin_data['Version'];

                wp_enqueue_script(
                    'falling-emojis-js',
                    plugins_url('assets/falling-emojis.js', __FILE__),
                    [],
                    $plugin_version, // Use the dynamic version here
                    true
                );

                // Pass ajaxurl to the JavaScript file
                $inline_script = 'const ajaxurl = "' . admin_url('admin-ajax.php') . '";';
                wp_add_inline_script('falling-emojis-js', $inline_script, 'before');
            }


            /**
             * Handle AJAX request for settings.
             */
            public function handle_ajax_settings() {
                $options = get_option('falling_emojis_options', [
                    'emoji_list' => '🌟,🌈,🍂,😄,🚀',
                    'speed' => 200,
                    'density' => 1,
                    'enabled' => 1,
                ]);
            
                wp_send_json([
                    'emoji_list' => explode(',', $options['emoji_list']),
                    'speed' => intval($options['speed']),
                    'density' => intval($options['density']),
                    'enabled' => (bool) $options['enabled'],
                ]);
            }
            

            /**
             * Register the submenu under the FLW Plugins menu.
             */
            public function register_submenu() {
                $pluginSlug = basename(dirname(__FILE__));
                FLW_Plugin_Library::add_submenu(
                    'Falling Emojis', // Title
                    $pluginSlug, // Slug
                    [$this, 'render_settings_page'] // Callback function
                );
            }

            /**
             * Render the settings page content.
             */
            public function render_settings_page() {
                echo '<div class="wrap">';
                echo '<h1>Falling Emojis Settings</h1>';
                echo '<form method="post" action="options.php">';
                settings_fields('falling_emojis_settings');
                do_settings_sections('falling-emojis-settings');
                submit_button();
                echo '</form>';
                echo '</div>';
            }

            /**
             * Register settings, sections, and fields.
             */
            public function register_settings() {
                register_setting('falling_emojis_settings', 'falling_emojis_options');
            
                add_settings_section(
                    'falling_emojis_main_section',
                    'Main Settings',
                    null,
                    'falling-emojis-settings'
                );
            
                add_settings_field(
                    'falling_emojis_list',
                    'Emoji List',
                    [$this, 'render_emoji_list_field'],
                    'falling-emojis-settings',
                    'falling_emojis_main_section'
                );
            
                add_settings_field(
                    'falling_emojis_speed',
                    'Falling Speed (ms)',
                    [$this, 'render_speed_field'],
                    'falling-emojis-settings',
                    'falling_emojis_main_section'
                );
            
                add_settings_field(
                    'falling_emojis_density',
                    'Density (Items Per Interval)',
                    [$this, 'render_density_field'],
                    'falling-emojis-settings',
                    'falling_emojis_main_section'
                );
            
                add_settings_field(
                    'falling_emojis_enabled',
                    'Enable Falling Emojis',
                    [$this, 'render_enabled_field'],
                    'falling-emojis-settings',
                    'falling_emojis_main_section'
                );
            }
            
            /**
             * Render the enable/disable checkbox field.
             */
            public function render_enabled_field() {
                $options = get_option('falling_emojis_options');
                $enabled = isset($options['enabled']) ? (bool) $options['enabled'] : false;
                echo '<input type="checkbox" name="falling_emojis_options[enabled]" value="1"' . checked(1, $enabled, false) . '> Enable the falling emojis effect';
            }
            

            /**
             * Render the emoji list field.
             */
            public function render_emoji_list_field() {
                $options = get_option('falling_emojis_options');
                $emoji_list = isset($options['emoji_list']) ? esc_attr($options['emoji_list']) : '';
                echo '<input type="text" name="falling_emojis_options[emoji_list]" value="' . $emoji_list . '" placeholder="Enter emojis separated by commas" style="width: 300px;">';
            }

            /**
             * Render the speed field.
             */
            public function render_speed_field() {
                $options = get_option('falling_emojis_options');
                $speed = isset($options['speed']) ? esc_attr($options['speed']) : '200';
                echo '<input type="number" name="falling_emojis_options[speed]" value="' . $speed . '" placeholder="200" style="width: 300px;">';
            }

            /**
             * Render the density field.
             */
            public function render_density_field() {
                $options = get_option('falling_emojis_options');
                $density = isset($options['density']) ? esc_attr($options['density']) : '1';
                echo '<input type="number" name="falling_emojis_options[density]" value="' . $density . '" placeholder="1" style="width: 300px;">';
            }
        }

        // Initialize the plugin
        new FallingEmojis_Plugin();
    } else {
        add_action('admin_notices', function () {
            echo '<div class="notice notice-error"><p>The FLW Plugin Library must be activated for Falling Emojis to work.</p></div>';
        });
    }
});
